// $Id: OEObject.h,v 1.2 1997/10/23 02:06:08 art Exp $

//
// ----------------------- OEObject Class Interface ---------------------------
//
// Original class (ObjectError) by:
// Bill Bumgarner, Andrew Stone, Mike Morton, and Julie Zelenski
//
// A class that poses as object and does crash-reporting.
// Catches terminating signals (ie seg faults, bus errors)
// and fatal Objective-C runtime errors and writes a backtrace
// out to the console using some shady hacks.  This could be modified
// to write backtrace to a file, mail message, etc... if desired.
//
// Modified by Ivo Rothschild (ivo@hasc.ca) to try to continue
//			when muddling on.
//
// Attempted port to OpenStep by Subrata Sircar (subrata_sircar@next.com).
//
// Attempted enhancements for OS/NT by Art Isbell (arti@lava.net).
//
// printf format repairs by Carl Lindberg (Carl_Lindberg@BLaCKSMITH.com).
//
// Win32 exception handling by Jeremy Bettis (jeremy@hksys.com).
//
// Use at own Risk (so sayeth the authors).
//
// NO WARRANTY:
// ANY IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE
// IS HEREBY DISCLAIMED.  IN NO EVENT WILL THE AFOREMENTIONED PARTIES BE LIABLE
// FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF OR INABILITY TO USE THIS CODE.
// ----------------------------------------------------------------------------
//

// ----------------------------- Header Files ---------------------------------

#import <Foundation/NSObject.h>

// ---------------- Typedef, Struct, and Union Declarations -------------------

// --------------------- Constant and Enum Definitions ------------------------

@interface OEObject:NSObject
{
// -------------------- Instance Variable Declarations ------------------------
@private
@protected
@public
}
// ----------------------- Class Method Declarations --------------------------

+ (void)initialize;
+ (void)setup;
+ (void)setContinueAfterError:(BOOL)aFlag;
+ (void)resumeHandlingCrashes;
+ (void)stopHandlingCrashes;
+ (NSString *)printBacktrace:(NSString *)aMessage
           skippedFrameCount:(unsigned)aCount;
#ifdef WIN32
+ (NSString *)printBacktrace:(NSString *)aMessage
                     address:(void *)aFramePointer;
#endif WIN32
+ (NSString *)crashException;
+ (NSString *)functionArgCountDefaultsName;
+ (NSString *)frameCountDefaultsName;
+ (NSString *)maxArgDescriptionLengthDefaultsName;

// ---------------- Overridden Instance Method Declarations -------------------

- (void)doesNotRecognizeSelector:(SEL)aSelector;

// ------------------- New Instance Method Declarations -----------------------

// ----------------- Delegate Instance Method Declarations --------------------

@end
