//************************************************************************
//
//	OICNSTableColumnInspector.m
//
//	by Juergen Moellenhoff <jm@oic.de>		
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "OICNSTableColumnInspector.h"
#import <OICInterface/OICNSTableHeaderView.h>

#import <InterfaceBuilder/IBInspectorManager.h>

static	NSString *dataCellModeIdentifier = @"OICDataCellInspectorMode";
static	NSString *inspectorModeIdentifier = @"OICNSTableColumnInspectorMode";

@implementation OICNSTableColumnInspector

+ (void)addInspectorForObject:object
{
	// Much easier with OPENSTEP 4.2
	[[IBInspectorManager sharedInspectorManager] 
		addInspectorModeWithIdentifier:inspectorModeIdentifier
		forObject:object
		localizedLabel:NSLocalizedStringFromTableInBundle(@"OICCOLUMN", @"OICInspectorMode", 
							[NSBundle bundleForClass:[self class]], nil)
		inspectorClassName:NSStringFromClass([OICNSTableColumnInspector class])
		ordering:2.0];
}

+ (void)addInspectorForDataCell:aCell
{
	NSString	*cellInspectorClassName;

	// Much easier with OPENSTEP 4.2
	if ((cellInspectorClassName = [aCell inspectorClassName]) == nil)
	{
		if ([aCell isKindOfClass:[NSImageCell class]])
			cellInspectorClassName = @"IBImageViewInspector";

		if ([aCell isKindOfClass:[NSTextFieldCell class]])
			cellInspectorClassName = @"IBTextFieldInspector";
	}

	if (cellInspectorClassName == nil)
		return;

	[[IBInspectorManager sharedInspectorManager] 
			addInspectorModeWithIdentifier:dataCellModeIdentifier
			forObject:aCell
			localizedLabel:NSLocalizedStringFromTableInBundle(@"OICDATACELL", @"OICInspectorMode", 
								[NSBundle bundleForClass:[self class]], nil)
			inspectorClassName:cellInspectorClassName
			ordering:2.5];
}

+ (void)removeInspectorForDataCell
{
	// Much easier with OPENSTEP 4.2
	[[IBInspectorManager sharedInspectorManager] removeInspectorModeWithIdentifier:dataCellModeIdentifier];
}

- init
{
	NSString 	*nibPath;
	
	[super init];
	
	nibPath = [[NSBundle bundleForClass:[self class]] 
				pathForResource:@"OICNSTableColumnInspector" ofType:@"nib"];

	[NSBundle loadNibFile:nibPath 
				externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil]
				withZone:[self zone]];
	
	 return self;
}

- (void)ok:sender
{
	if (sender == orderingMatrix)
	{
        switch([[orderingMatrix selectedCell] tag])
        {
        case 0:
            [[self object] setOrdering:NSOrderedAscending];
            break;

        case 1:
            [[self object] setOrdering:NSOrderedDescending];
            break;

        case 2:
        default:
            [[self object] setOrdering:NSOrderedSame];
            break;
        }

        [[NSApp activeDocument] drawObject:[self object]];
	}

	if (sender == dataCellMatrix)
	{
		id	cell = nil;

        switch([[dataCellMatrix selectedCell] tag])
        {
        case 1: // NSImageCell
            cell = [[NSImageCell alloc] init];
            break;

        case 0: // NSTextFieldCell
        default:
            cell = [[NSTextFieldCell alloc] init];
            break;
        }

		[[NSApp activeDocument] detachObject:[[self object] dataCell]];

		[[self object] setDataCell:cell];
		[cell release];

		// Much easier with OPENSTEP 4.2
		[OICNSTableColumnInspector removeInspectorForDataCell];
		[OICNSTableColumnInspector addInspectorForDataCell:[[self object] dataCell]];
	}

	if (sender == caseInsensitiveSwitch)
		[[self object] setCaseInsensitive:([caseInsensitiveSwitch state] != 0)];

	[super ok:sender];
}


- (void)revert:sender
{
	int	selectedCell = 0;
	id	dataCell;

	[caseInsensitiveSwitch setState:[[self object] isCaseInsensitive]];

	switch([(OICNSTableColumn *)[self object] ordering])
	{
	case NSOrderedAscending:
		selectedCell = 0;
		break;

	case NSOrderedDescending:
		selectedCell = 1;
		break;

	case NSOrderedSame:
	default:
		selectedCell = 2;
		break;
	}
	[orderingMatrix selectCellWithTag:selectedCell];


	dataCell = [[self object] dataCell];
	if ([dataCell isKindOfClass:[NSTextFieldCell class]])
		selectedCell = 0;

	if ([dataCell isKindOfClass:[NSImageCell class]])
		selectedCell = 1;

	[dataCellMatrix selectCellWithTag:selectedCell];
	[super revert:sender];
}


- (BOOL)wantsButtons
{
	return NO;
}

@end

@implementation OICNSTableColumn(C_OICNSTableColumnIBAdds)

- (void)setTableView:(NSTableView *)aTableView
{
	[super setTableView:aTableView];

	// Create the right headerView when someone copy, paste the OICNSTableColumn
	// into a "normal" NSTableView.
	if (![[aTableView headerView] isKindOfClass:[OICNSTableHeaderView class]])
	{
		OICNSTableHeaderView	*newTableHeaderView;

		newTableHeaderView = [[[OICNSTableHeaderView alloc] initWithFrame:[[aTableView headerView] frame]] autorelease];

		NSLog(@"%@:-%@, Replace HeaderView [%@] with HeaderView [%@]", 
			NSStringFromClass([self class]), NSStringFromSelector(_cmd), 
			[aTableView headerView], newTableHeaderView);

		[aTableView setHeaderView:newTableHeaderView];
	}
}

- awakeAfterUsingCoder:(NSCoder *)aDecoder
{
 	// ...set the fetch order in IB test mode...
	[self performSelector:@selector(awakeFromNib) withObject:self afterDelay:250.0 / 1000.0];

	return self;
}

@end

@implementation IBInspectorManager(C_IBInspectorManagerAdds)

// Much easier with OPENSTEP 4.2
- (void)removeInspectorModeWithIdentifier:(NSString *)identifier
{
	unsigned int	index;

	index = [self indexOfModeWithIdentifier:identifier];
	if (index != NSNotFound)
		[modes removeObjectAtIndex:index];
}

@end
