//************************************************************************
//
//	OICTableView.m
//
//	by Juergen Moellenhoff <jm@oic.de>		
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "OICTableView.h"
#import "OICNSTableColumnInspector.h"

#import <OICInterface/OICNSTableViewPrivate.h>
#import <OICInterface/OICNSTableHeaderView.h>

@implementation OICTableView

- (void)_willInspectObject:(NSNotification *)notification
{
	// Thanks to Derek Clegg <Derek_Clegg@next.com> for the hint.
	if ([[notification object] isKindOfClass:[OICNSTableColumn class]])
	{
		// Much easier with OPENSTEP 4.2
		[OICNSTableColumnInspector addInspectorForObject:[notification object]];
		[OICNSTableColumnInspector addInspectorForDataCell:[[notification object] dataCell]];
	}
}

- init
{
	[super init];
	
	// Thanks to Derek Clegg <Derek_Clegg@next.com> for the hint.
	[[NSNotificationCenter defaultCenter]
		addObserver:self 
		selector:@selector(_willInspectObject:)
		name:IBWillInspectObjectNotification
		object:nil];

	return self;
}

- (void)finishInstantiate 
{
	unsigned int		i, iCount;
	NSTableColumn		*oldColumn;
	OICNSTableColumn	*newColumn;
	NSArray				*colArray;

	// Replace headerView
	[tableView setHeaderView:[[[OICNSTableHeaderView alloc] initWithFrame:[[tableView headerView] frame]] autorelease]];

	// Remove the old NSTableColumns from the palette
	colArray = [NSArray arrayWithArray:[tableView tableColumns]];

	// Part of IBDocument protocol
	// Thanks to Kenny Leung <Kenny_Leung@next.com> for the hint.
	[[self paletteDocument] detachObjects:colArray];

	for (i = 0, iCount = [colArray count]; i < iCount; i++)
		[tableView removeTableColumn:[colArray objectAtIndex:i]];

	// Add the new OICNSTableColumn to the palette
	for (i = 0, iCount = [colArray count]; i < iCount; i++)
	{
		oldColumn = [colArray objectAtIndex:i];

		newColumn = [OICNSTableView _createNewTableColumn:oldColumn];
		[newColumn setHeaderCell:(id)[OICNSTableColumn _createNewHeaderCell:[oldColumn headerCell]]];

		[tableView addTableColumn:newColumn];

		// Part of IBDocument protocol
		// Thanks to Kenny Leung <Kenny_Leung@next.com> for the hint.
       [[self paletteDocument] attachObject:newColumn toParent:tableView];
	}
}

@end
